@@ -34,7 +34,7 @@ except ImportError:
 import tencentcloud
 from tencentcloud.common.exception.tencent_cloud_sdk_exception import TencentCloudSDKException
 from tencentcloud.common.exception import TencentCloudSDKException as SDKError
-from tencentcloud.common.http.request import ApiRequest
+from tencentcloud.common.http.request import ApiRequest, ResponsePrettyFormatter
 from tencentcloud.common.http.request import RequestInternal
 from tencentcloud.common.profile.client_profile import ClientProfile, RegionBreakerProfile
 from tencentcloud.common.sign import Sign
@@ -77,7 +77,8 @@ class AbstractClient(object):
                                   req_timeout=self.profile.httpProfile.reqTimeout,
                                   proxy=self.profile.httpProfile.proxy,
                                   is_http=is_http,
-                                  certification=self.profile.httpProfile.certification)
+                                  certification=self.profile.httpProfile.certification,
+                                  pre_conn_pool_size=self.profile.httpProfile.pre_conn_pool_size)
         if self.profile.httpProfile.keepAlive:
             self.request.set_keep_alive()
         self.circuit_breaker = None
@@ -85,6 +86,10 @@ class AbstractClient(object):
             if self.profile.region_breaker_profile is None:
                 self.profile.region_breaker_profile = RegionBreakerProfile()
             self.circuit_breaker = CircuitBreaker(self.profile.region_breaker_profile)
+        if self.profile.request_client:
+            self.request_client = self._sdkVersion + "; " + self.profile.request_client
+        else:
+            self.request_client = self._sdkVersion
 
     def _fix_params(self, params):
         if not isinstance(params, (dict,)):
@@ -134,7 +139,7 @@ class AbstractClient(object):
     def _build_req_with_old_signature(self, action, params, req):
         params = copy.deepcopy(self._fix_params(params))
         params['Action'] = action[0].upper() + action[1:]
-        params['RequestClient'] = self._sdkVersion
+        params['RequestClient'] = self.request_client
         params['Nonce'] = random.randint(1, sys.maxsize)
         params['Timestamp'] = int(time.time())
         params['Version'] = self._apiVersion
@@ -183,7 +188,7 @@ class AbstractClient(object):
         timestamp = int(time.time())
         req.header["Host"] = endpoint
         req.header["X-TC-Action"] = action[0].upper() + action[1:]
-        req.header["X-TC-RequestClient"] = self._sdkVersion
+        req.header["X-TC-RequestClient"] = self.request_client
         req.header["X-TC-Timestamp"] = str(timestamp)
         req.header["X-TC-Version"] = self._apiVersion
         if self.profile.unsignedPayload is True:
@@ -274,7 +279,7 @@ class AbstractClient(object):
         timestamp = int(time.time())
         req.header["Host"] = endpoint
         req.header["X-TC-Action"] = action[0].upper() + action[1:]
-        req.header["X-TC-RequestClient"] = self._sdkVersion
+        req.header["X-TC-RequestClient"] = self.request_client
         req.header["X-TC-Timestamp"] = str(timestamp)
         req.header["X-TC-Version"] = self._apiVersion
         if self.profile.unsignedPayload is True:
@@ -324,6 +329,7 @@ class AbstractClient(object):
 
     def _check_status(self, resp_inter):
         if resp_inter.status_code != 200:
+            logger.debug("GetResponse: %s", ResponsePrettyFormatter(resp_inter))
             raise TencentCloudSDKException("ServerNetworkError", resp_inter.content)
 
     def _format_sign_string(self, params):
@@ -354,6 +360,7 @@ class AbstractClient(object):
             code = data["Response"]["Error"]["Code"]
             message = data["Response"]["Error"]["Message"]
             reqid = data["Response"]["RequestId"]
+            logger.debug("GetResponse: %s", ResponsePrettyFormatter(resp))
             raise TencentCloudSDKException(code, message, reqid)
         if "DeprecatedWarning" in data["Response"]:
             import warnings
@@ -363,6 +370,7 @@ class AbstractClient(object):
 
     @staticmethod
     def _process_response_sse(resp):
+        logger.debug("GetResponse: %s", ResponsePrettyFormatter(resp, format_body=False))
         e = {}
 
         for line in resp.iter_lines():
@@ -371,6 +379,7 @@ class AbstractClient(object):
                 e = {}
                 continue
 
+            logger.debug("GetResponse: %s", line)
             line = line.decode('utf-8')
 
             # comment
@@ -392,6 +401,12 @@ class AbstractClient(object):
                 e[key] = int(val)
 
     def _call(self, action, params, options=None, headers=None):
+        if headers is None:
+            headers = {}
+        if not isinstance(headers, dict):
+            raise TencentCloudSDKException("ClientError", "headers must be a dict.")
+        if "x-tc-traceid" not in {k.lower() for k in headers.keys()}:
+            headers["X-TC-TraceId"] = str(uuid.uuid4())
         if not self.profile.disable_region_breaker:
             return self._call_with_region_breaker(action, params, options, headers)
         req = RequestInternal(self._get_endpoint(),
@@ -400,12 +415,16 @@ class AbstractClient(object):
                               header=headers)
         self._build_req_inter(action, params, req, options)
 
+        if self.profile.httpProfile.apigw_endpoint:
+            req.host = self.profile.httpProfile.apigw_endpoint
+            req.header["Host"] = req.host
         return self.request.send_request(req)
 
     def call(self, action, params, options=None, headers=None):
         resp = self._call(action, params, options, headers)
         self._check_status(resp)
         self._check_error(resp)
+        logger.debug("GetResponse: %s", ResponsePrettyFormatter(resp))
         return resp.content
 
     def _call_with_region_breaker(self, action, params, options=None, headers=None):
@@ -485,6 +504,7 @@ class AbstractClient(object):
         resp = self._call(action, params, options, headers)
         self._check_status(resp)
         self._check_error(resp)
+        logger.debug("GetResponse: %s", ResponsePrettyFormatter(resp))
         return json.loads(resp.content)
 
     def call_sse(self, action, params, headers=None, options=None):
